#!/usr/bin/env python3
# An example of implementing integer factorisation in Python: find all prime numbers that divide a given number

# prime number test
def isPrime(n):
    """Test whether the given number @n@ is a prime number."""
    if n == 1:
        return False
    if n == 2:
        return True
    if n % 2 == 0:
        return False
    for x in range(3,n,2):
        if n % x == 0:
            return False
    else:
        return True

# build a list of all prime numbers, to be used later
# NB: this is inefficient since this repeatedly checks for divisability of lower numbers
primes = [ n for n in range(3,39,2) if isPrime(n) ]
primes.insert(0,2)

def factor(m : int) -> int:
    """For a given integer number @n@, find all prime numbers that divide it."""
    res = []
    for x in primes:
        if x > m:
            break
        while m % x == 0:
            res.append(x)
            m = m // x
    return res
    
# This function turns a list with repeated entries into a list of pairs of (value, # of occurences)
def cleanup(xs):
    n = 0
    x = 0
    ys = []
    while (xs != []):
        if (x == xs[0]):
            n += 1
            del xs[0]
        else:
            if n > 0:
                ys.append((x,n))
            else:
                pass
            x = xs[0]
            del xs[0]
            n = 1
    if n > 0:
        ys.append((x,n))
    return ys   


# Simple provided test() function used in main() to print
# what each function returns vs. what it's supposed to return.
def test(got, expected):
  if got == expected:
    prefix = ' OK '
  else:
    prefix = ' ** '
  print('%s got: %s expected: %s' % (prefix, repr(got), repr(expected)))

if __name__ == '__main__':
  print("Integer factorisation:")
  test(factor(120),[2, 2, 2,3, 5])
  test(cleanup(factor(120)), [(2, 3), (3, 1), (5, 1)])

